@@ -21,6 +21,7 @@
 #include <libkleo/dn.h>
 #include <libkleo/enum.h>
 #include <libkleo/filesystemwatcher.h>
+#include <libkleo/gnupg.h>
 #include <libkleo/keygroup.h>
 #include <libkleo/keygroupconfig.h>
 #include <libkleo/keyhelpers.h>
@@ -58,7 +59,6 @@
 using namespace std::chrono_literals;
 using namespace Kleo;
 using namespace GpgME;
-using namespace KMime::Types;
 
 static const unsigned int hours2ms = 1000 * 60 * 60;
 
@@ -179,6 +179,11 @@ public:
 
     std::vector<Key> find_mailbox(const QString &email, bool sign) const;
 
+    std::vector<Subkey>::const_iterator find_subkeyfpr(const char *subkeyfpr) const
+    {
+        return find<_detail::BySubkeyFingerprint>(by.subkeyfpr, subkeyfpr);
+    }
+
     std::vector<Subkey>::const_iterator find_keygrip(const char *keygrip) const
     {
         return find<_detail::ByKeyGrip>(by.keygrip, keygrip);
@@ -194,11 +199,6 @@ public:
         return find<_detail::ByKeyID>(by.keyid, keyid);
     }
 
-    std::vector<Key>::const_iterator find_shortkeyid(const char *shortkeyid) const
-    {
-        return find<_detail::ByShortKeyID>(by.shortkeyid, shortkeyid);
-    }
-
     std::pair<std::vector<Key>::const_iterator, std::vector<Key>::const_iterator> find_subjects(const char *chain_id) const
     {
         ensureCachePopulated();
@@ -427,9 +427,9 @@ private:
     int m_refreshInterval;
 
     struct By {
-        std::vector<Key> fpr, keyid, shortkeyid, chainid;
+        std::vector<Key> fpr, keyid, chainid;
         std::vector<std::pair<std::string, Key>> email;
-        std::vector<Subkey> subkeyid, keygrip;
+        std::vector<Subkey> subkeyfpr, subkeyid, keygrip;
     } by;
     bool m_initalized;
     bool m_pgpOnly;
@@ -437,6 +437,7 @@ private:
     bool m_groupsEnabled = false;
     std::shared_ptr<KeyGroupConfig> m_groupConfig;
     std::vector<KeyGroup> m_groups;
+    std::unordered_map<QByteArray, std::vector<CardKeyStorageInfo>> m_cards;
 };
 
 std::shared_ptr<const KeyCache> KeyCache::instance()
@@ -501,20 +502,30 @@ std::shared_ptr<KeyCacheAutoRefreshSuspension> KeyCache::suspendAutoRefresh()
     return KeyCacheAutoRefreshSuspension::instance();
 }
 
-void KeyCache::reload(GpgME::Protocol /*proto*/)
+void KeyCache::reload(GpgME::Protocol /*proto*/, ReloadOption option)
 {
-    if (d->m_refreshJob) {
+    qCDebug(LIBKLEO_LOG) << this << __func__ << "option:" << option;
+    const bool forceReload = option & ForceReload;
+    if (d->m_refreshJob && !forceReload) {
+        qCDebug(LIBKLEO_LOG) << this << __func__ << "- refresh already running";
         return;
     }
+    if (d->m_refreshJob) {
+        disconnect(d->m_refreshJob.data(), nullptr, this, nullptr);
+        d->m_refreshJob->cancel();
+        d->m_refreshJob.clear();
+    }
 
     d->updateAutoKeyListingTimer();
 
     enableFileSystemWatcher(false);
     d->m_refreshJob = new RefreshKeysJob(this);
     connect(d->m_refreshJob.data(), &RefreshKeysJob::done, this, [this](const GpgME::KeyListResult &r) {
+        qCDebug(LIBKLEO_LOG) << d->m_refreshJob.data() << "RefreshKeysJob::done";
         d->refreshJobDone(r);
     });
     connect(d->m_refreshJob.data(), &RefreshKeysJob::canceled, this, [this]() {
+        qCDebug(LIBKLEO_LOG) << d->m_refreshJob.data() << "RefreshKeysJob::canceled";
         d->m_refreshJob.clear();
     });
     d->m_refreshJob->start();
@@ -628,21 +639,6 @@ std::vector<Key> KeyCache::findByEMailAddress(const std::string &email) const
     return findByEMailAddress(email.c_str());
 }
 
-const Key &KeyCache::findByShortKeyID(const char *id) const
-{
-    const std::vector<Key>::const_iterator it = d->find_shortkeyid(id);
-    if (it != d->by.shortkeyid.end()) {
-        return *it;
-    }
-    static const Key null;
-    return null;
-}
-
-const Key &KeyCache::findByShortKeyID(const std::string &id) const
-{
-    return findByShortKeyID(id.c_str());
-}
-
 const Key &KeyCache::findByKeyIDOrFingerprint(const char *id) const
 {
     {
@@ -689,7 +685,7 @@ std::vector<Key> KeyCache::findByKeyIDOrFingerprint(const std::vector<std::strin
                               std::back_inserter(result),
                               _detail::ByFingerprint<std::less>());
     if (result.size() < keyids.size()) {
-        // note that By{Fingerprint,KeyID,ShortKeyID} define the same
+        // note that By{Fingerprint,KeyID} define the same
         // order for _strings_
         kdtools::set_intersection(d->by.keyid.begin(),
                                   d->by.keyid.end(),
@@ -732,6 +728,28 @@ const Subkey &KeyCache::findSubkeyByKeyGrip(const std::string &grip, Protocol pr
     return findSubkeyByKeyGrip(grip.c_str(), protocol);
 }
 
+std::vector<GpgME::Subkey> Kleo::KeyCache::findSubkeysByKeyGrip(const char *grip, GpgME::Protocol protocol) const
+{
+    d->ensureCachePopulated();
+
+    std::vector<GpgME::Subkey> subkeys;
+    const auto range = std::equal_range(d->by.keygrip.begin(), d->by.keygrip.end(), grip, _detail::ByKeyGrip<std::less>());
+    subkeys.reserve(std::distance(range.first, range.second));
+    if (protocol == UnknownProtocol) {
+        std::copy(range.first, range.second, std::back_inserter(subkeys));
+    } else {
+        std::copy_if(range.first, range.second, std::back_inserter(subkeys), [protocol](const auto &subkey) {
+            return subkey.parent().protocol() == protocol;
+        });
+    }
+    return subkeys;
+}
+
+std::vector<GpgME::Subkey> Kleo::KeyCache::findSubkeysByKeyGrip(const std::string &grip, GpgME::Protocol protocol) const
+{
+    return findSubkeysByKeyGrip(grip.c_str(), protocol);
+}
+
 std::vector<Subkey> KeyCache::findSubkeysByKeyID(const std::vector<std::string> &ids) const
 {
     std::vector<std::string> sorted;
@@ -753,6 +771,17 @@ std::vector<Subkey> KeyCache::findSubkeysByKeyID(const std::vector<std::string>
     return result;
 }
 
+const GpgME::Subkey &KeyCache::findSubkeyByFingerprint(const std::string &fpr) const
+{
+    static const Subkey null;
+
+    const auto it = d->find_subkeyfpr(fpr.c_str());
+    if (it != d->by.subkeyfpr.end()) {
+        return *it;
+    }
+    return null;
+}
+
 std::vector<Key> KeyCache::findRecipients(const DecryptionResult &res) const
 {
     std::vector<std::string> keyids;
@@ -772,16 +801,36 @@ std::vector<Key> KeyCache::findRecipients(const DecryptionResult &res) const
     return result;
 }
 
-std::vector<Key> KeyCache::findSigners(const VerificationResult &res) const
+GpgME::Key KeyCache::findSigner(const GpgME::Signature &signature) const
 {
-    std::vector<std::string> fprs;
-    const auto signatures = res.signatures();
-    for (const Signature &s : signatures) {
-        if (const char *fpr = s.fingerprint()) {
-            fprs.push_back(fpr);
+    if (signature.isNull()) {
+        return {};
+    }
+
+    GpgME::Key key = signature.key();
+    if (key.isNull() && signature.fingerprint()) {
+        key = findByFingerprint(signature.fingerprint());
+    }
+    if (key.isNull() && signature.fingerprint()) {
+        // try to find a subkey that was used for signing
+        const auto subkey = findSubkeyByFingerprint(signature.fingerprint());
+        if (!subkey.isNull()) {
+            key = subkey.parent();
         }
     }
-    return findByKeyIDOrFingerprint(fprs);
+    return key;
+}
+
+std::vector<Key> KeyCache::findSigners(const VerificationResult &res) const
+{
+    std::vector<Key> signers;
+    if (res.numSignatures() > 0) {
+        signers.reserve(res.numSignatures());
+        Kleo::transform(res.signatures(), std::back_inserter(signers), [this](const auto &sig) {
+            return findSigner(sig);
+        });
+    }
+    return signers;
 }
 
 std::vector<Key> KeyCache::findSigningKeysByMailbox(const QString &mb) const
@@ -808,11 +857,7 @@ struct ready_for_signing {
     bool operator()(const Key &key) const
     {
         ACCEPT(hasSecret);
-#if GPGMEPP_KEY_HAS_HASCERTIFY_SIGN_ENCRYPT_AUTHENTICATE
         ACCEPT(hasSign);
-#else
-        ACCEPT(canSign);
-#endif
         REJECT(isRevoked);
         REJECT(isExpired);
         REJECT(isDisabled);
@@ -831,20 +876,12 @@ struct ready_for_signing {
 struct ready_for_encryption {
     bool operator()(const Key &key) const
     {
-#if 1
-#if GPGMEPP_KEY_HAS_HASCERTIFY_SIGN_ENCRYPT_AUTHENTICATE
         ACCEPT(hasEncrypt);
-#else
-        ACCEPT(canEncrypt);
-#endif
         REJECT(isRevoked);
         REJECT(isExpired);
         REJECT(isDisabled);
         REJECT(isInvalid);
         return true;
-#else
-        return key.hasEncrypt() && !key.isRevoked() && !key.isExpired() && !key.isDisabled() && !key.isInvalid();
-#endif
     }
 #undef DO
 #undef ACCEPT
@@ -872,42 +909,49 @@ std::vector<Key> KeyCache::Private::find_mailbox(const QString &email, bool sign
 
 std::vector<Key> KeyCache::findSubjects(const GpgME::Key &key, Options options) const
 {
+    if (key.isNull()) {
+        return {};
+    }
+
     return findSubjects(std::vector<Key>(1, key), options);
 }
 
 std::vector<Key> KeyCache::findSubjects(const std::vector<Key> &keys, Options options) const
 {
-    return findSubjects(keys.begin(), keys.end(), options);
-}
-
-std::vector<Key> KeyCache::findSubjects(std::vector<Key>::const_iterator first, std::vector<Key>::const_iterator last, Options options) const
-{
-    if (first == last) {
-        return std::vector<Key>();
-    }
-
     std::vector<Key> result;
-    while (first != last) {
-        const auto pair = d->find_subjects(first->primaryFingerprint());
-        result.insert(result.end(), pair.first, pair.second);
-        ++first;
+
+    if (keys.empty()) {
+        return result;
     }
 
-    std::sort(result.begin(), result.end(), _detail::ByFingerprint<std::less>());
-    result.erase(std::unique(result.begin(), result.end(), _detail::ByFingerprint<std::equal_to>()), result.end());
+    // get the immediate subjects
+    for (const auto &key : keys) {
+        const auto firstAndLastSubject = d->find_subjects(key.primaryFingerprint());
+        result.insert(result.end(), firstAndLastSubject.first, firstAndLastSubject.second);
+    }
+    // remove duplicates
+    _detail::sort_by_fpr(result);
+    _detail::remove_duplicates_by_fpr(result);
 
     if (options & RecursiveSearch) {
-        const std::vector<Key> furtherSubjects = findSubjects(result, options);
-        std::vector<Key> combined;
-        combined.reserve(result.size() + furtherSubjects.size());
-        std::merge(result.begin(),
-                   result.end(),
-                   furtherSubjects.begin(),
-                   furtherSubjects.end(),
-                   std::back_inserter(combined),
-                   _detail::ByFingerprint<std::less>());
-        combined.erase(std::unique(combined.begin(), combined.end(), _detail::ByFingerprint<std::equal_to>()), combined.end());
-        result.swap(combined);
+        for (std::vector<Key> furtherSubjects = findSubjects(result, NoOption); //
+             !furtherSubjects.empty();
+             furtherSubjects = findSubjects(furtherSubjects, NoOption)) {
+            std::vector<Key> combined;
+            combined.reserve(result.size() + furtherSubjects.size());
+            std::merge(result.begin(),
+                       result.end(),
+                       furtherSubjects.begin(),
+                       furtherSubjects.end(),
+                       std::back_inserter(combined),
+                       _detail::ByFingerprint<std::less>());
+            _detail::remove_duplicates_by_fpr(combined);
+            if (result.size() == combined.size()) {
+                // no new subjects were found; this happens if a chain has a cycle
+                break;
+            }
+            result.swap(combined);
+        }
     }
 
     return result;
@@ -929,7 +973,7 @@ std::vector<Key> KeyCache::findIssuers(const Key &key, Options options) const
         return result;
     }
 
-    const Key &issuer = findByFingerprint(key.chainID());
+    Key issuer = findByFingerprint(key.chainID());
 
     if (issuer.isNull()) {
         return result;
@@ -941,8 +985,8 @@ std::vector<Key> KeyCache::findIssuers(const Key &key, Options options) const
         return result;
     }
 
-    while (true) {
-        const Key &issuer = findByFingerprint(result.back().chainID());
+    while (!issuer.isRoot()) {
+        issuer = findByFingerprint(result.back().chainID());
         if (issuer.isNull()) {
             break;
         }
@@ -950,9 +994,10 @@ std::vector<Key> KeyCache::findIssuers(const Key &key, Options options) const
             return _detail::ByFingerprint<std::equal_to>()(issuer, key);
         });
         // we also add the issuer if the chain already contains it, so that
-        // the user can spot the recursion
+        // the user can spot the cycle
         result.push_back(issuer);
-        if (issuer.isRoot() || chainAlreadyContainsIssuer) {
+        if (chainAlreadyContainsIssuer) {
+            // break on cycle in chain
             break;
         }
     }
@@ -1017,14 +1062,6 @@ void KeyCache::remove(const Key &key)
         d->by.keyid.erase(it, range.second);
     }
 
-    if (const char *shortkeyid = key.shortKeyID()) {
-        const auto range = std::equal_range(d->by.shortkeyid.begin(), d->by.shortkeyid.end(), shortkeyid, _detail::ByShortKeyID<std::less>());
-        const auto it = std::remove_if(range.first, range.second, [fpr](const GpgME::Key &key) {
-            return _detail::ByFingerprint<std::equal_to>()(fpr, key);
-        });
-        d->by.shortkeyid.erase(it, range.second);
-    }
-
     if (const char *chainid = key.chainID()) {
         const auto range = std::equal_range(d->by.chainid.begin(), d->by.chainid.end(), chainid, _detail::ByChainID<std::less>());
         const auto range2 = std::equal_range(range.first, range.second, fpr, _detail::ByFingerprint<std::less>());
@@ -1042,6 +1079,13 @@ void KeyCache::remove(const Key &key)
 
     const auto keySubKeys{key.subkeys()};
     for (const Subkey &subkey : keySubKeys) {
+        if (const char *subkeyfpr = subkey.fingerprint()) {
+            const auto range = std::equal_range(d->by.subkeyfpr.begin(), d->by.subkeyfpr.end(), subkeyfpr, _detail::BySubkeyFingerprint<std::less>());
+            const auto it = std::remove_if(range.first, range.second, [fpr](const Subkey &subkey) {
+                return !qstricmp(fpr, subkey.parent().primaryFingerprint());
+            });
+            d->by.subkeyfpr.erase(it, range.second);
+        }
         if (const char *keyid = subkey.keyID()) {
             const auto range = std::equal_range(d->by.subkeyid.begin(), d->by.subkeyid.end(), keyid, _detail::ByKeyID<std::less>());
             const auto it = std::remove_if(range.first, range.second, [fpr](const Subkey &subkey) {
@@ -1223,17 +1267,16 @@ struct lexicographically {
 
 void KeyCache::insert(const std::vector<Key> &keys)
 {
-    // 1. remove those with empty fingerprints:
+    // 1. filter out keys with empty fingerprints:
     std::vector<Key> sorted;
     sorted.reserve(keys.size());
-    std::remove_copy_if(keys.begin(), keys.end(), std::back_inserter(sorted), [](const Key &key) {
+    std::copy_if(keys.begin(), keys.end(), std::back_inserter(sorted), [](const Key &key) {
         auto fp = key.primaryFingerprint();
-        return !fp || !*fp;
+        return fp && *fp;
     });
 
-    Q_FOREACH (const Key &key, sorted) {
-        remove(key); // this is sub-optimal, but makes implementation from here on much easier
-    }
+    // this is sub-optimal, but makes implementation from here on much easier
+    remove(sorted);
 
     // 2. sort by fingerprint:
     std::sort(sorted.begin(), sorted.end(), _detail::ByFingerprint<std::less>());
@@ -1285,18 +1328,7 @@ void KeyCache::insert(const std::vector<Key> &keys)
     by_keyid.reserve(sorted.size() + d->by.keyid.size());
     std::merge(sorted.begin(), sorted.end(), d->by.keyid.begin(), d->by.keyid.end(), std::back_inserter(by_keyid), _detail::ByKeyID<std::less>());
 
-    // 5. sort by short key id:
-    std::sort(sorted.begin(), sorted.end(), _detail::ByShortKeyID<std::less>());
-
-    // 5a. insert into short keyid index:
-    std::vector<Key> by_shortkeyid;
-    by_shortkeyid.reserve(sorted.size() + d->by.shortkeyid.size());
-    std::merge(sorted.begin(),
-               sorted.end(),
-               d->by.shortkeyid.begin(),
-               d->by.shortkeyid.end(),
-               std::back_inserter(by_shortkeyid),
-               _detail::ByShortKeyID<std::less>());
+    // 5. has been removed
 
     // 6. build subkey ID index:
     std::vector<Subkey> subkeys;
@@ -1304,6 +1336,9 @@ void KeyCache::insert(const std::vector<Key> &keys)
     for (const Key &key : std::as_const(sorted)) {
         const auto keySubkeys{key.subkeys()};
         for (const Subkey &subkey : keySubkeys) {
+            if (subkey.canRenc()) {
+                continue;
+            }
             subkeys.push_back(subkey);
         }
     }
@@ -1324,11 +1359,24 @@ void KeyCache::insert(const std::vector<Key> &keys)
     by_keygrip.reserve(subkeys.size() + d->by.keygrip.size());
     std::merge(subkeys.begin(), subkeys.end(), d->by.keygrip.begin(), d->by.keygrip.end(), std::back_inserter(by_keygrip), _detail::ByKeyGrip<std::less>());
 
+    // 6e sort by fingerprint:
+    std::sort(subkeys.begin(), subkeys.end(), _detail::BySubkeyFingerprint<std::less>());
+
+    // 6f. insert into subkey fingerprint index:
+    std::vector<Subkey> by_subkeyfpr;
+    by_subkeyfpr.reserve(subkeys.size() + d->by.subkeyfpr.size());
+    std::merge(subkeys.begin(),
+               subkeys.end(),
+               d->by.subkeyfpr.begin(),
+               d->by.subkeyfpr.end(),
+               std::back_inserter(by_subkeyfpr),
+               _detail::BySubkeyFingerprint<std::less>());
+
     // now commit (well, we already removed keys...)
     by_fpr.swap(d->by.fpr);
     by_keyid.swap(d->by.keyid);
-    by_shortkeyid.swap(d->by.shortkeyid);
     by_email.swap(d->by.email);
+    by_subkeyfpr.swap(d->by.subkeyfpr);
     by_subkeyid.swap(d->by.subkeyid);
     by_keygrip.swap(d->by.keygrip);
     by_chainid.swap(d->by.chainid);
@@ -1337,6 +1385,31 @@ void KeyCache::insert(const std::vector<Key> &keys)
         d->m_pgpOnly &= key.protocol() == GpgME::OpenPGP;
     }
 
+    d->m_cards.clear();
+    for (const auto &key : keys) {
+        for (const auto &subkey : key.subkeys()) {
+            if (!subkey.isSecret() || !d->m_cards[QByteArray(subkey.keyGrip())].empty()) {
+                continue;
+            }
+            const auto data = readSecretKeyFile(QString::fromLatin1(subkey.keyGrip()));
+            for (const auto &line : data) {
+                if (line.startsWith(QByteArrayLiteral("Token"))) {
+                    const auto split = line.split(' ');
+                    if (split.size() > 2) {
+                        const auto keyRef = QString::fromUtf8(split[2]).trimmed();
+                        d->m_cards[QByteArray(subkey.keyGrip())].push_back(CardKeyStorageInfo{
+                            QString::fromUtf8(split[1]),
+                            split.size() > 4 ? QString::fromLatin1(
+                                QString::fromUtf8(split[4]).trimmed().replace(QLatin1Char('+'), QLatin1Char(' ')).toUtf8().percentDecoded())
+                                             : QString(),
+                            keyRef,
+                        });
+                    }
+                }
+            }
+        }
+    }
+
     Q_EMIT keysMayHaveChanged();
 }
 
@@ -1361,14 +1434,16 @@ public:
     Error startKeyListing(GpgME::Protocol protocol);
     void listAllKeysJobDone(const KeyListResult &res, const std::vector<Key> &nextKeys)
     {
-        std::vector<Key> keys;
-        keys.reserve(m_keys.size() + nextKeys.size());
-        if (m_keys.empty()) {
-            keys = nextKeys;
-        } else {
-            std::merge(m_keys.begin(), m_keys.end(), nextKeys.begin(), nextKeys.end(), std::back_inserter(keys), _detail::ByFingerprint<std::less>());
+        if (!nextKeys.empty()) {
+            std::vector<Key> keys;
+            keys.reserve(m_keys.size() + nextKeys.size());
+            if (m_keys.empty()) {
+                keys = nextKeys;
+            } else {
+                std::merge(m_keys.begin(), m_keys.end(), nextKeys.begin(), nextKeys.end(), std::back_inserter(keys), _detail::ByFingerprint<std::less>());
+            }
+            m_keys.swap(keys);
         }
-        m_keys.swap(keys);
         jobDone(res);
     }
     void emitDone(const KeyListResult &result);
@@ -1403,10 +1478,10 @@ void KeyCache::RefreshKeysJob::Private::jobDone(const KeyListResult &result)
     if (sender) {
         sender->disconnect(q);
     }
-    Q_ASSERT(m_jobsPending.size() > 0);
+    Q_ASSERT(!m_jobsPending.empty());
     m_jobsPending.removeOne(qobject_cast<QGpgME::ListAllKeysJob *>(sender));
     m_mergedResult.mergeWith(result);
-    if (m_jobsPending.size() > 0) {
+    if (!m_jobsPending.empty()) {
         return;
     }
     updateKeyCache();
@@ -1432,6 +1507,7 @@ KeyCache::RefreshKeysJob::~RefreshKeysJob()
 
 void KeyCache::RefreshKeysJob::start()
 {
+    qCDebug(LIBKLEO_LOG) << "KeyCache::RefreshKeysJob" << __func__;
     QTimer::singleShot(0, this, [this]() {
         d->doStart();
     });
@@ -1451,11 +1527,11 @@ void KeyCache::RefreshKeysJob::Private::doStart()
         return;
     }
 
-    Q_ASSERT(m_jobsPending.size() == 0);
+    Q_ASSERT(m_jobsPending.empty());
     m_mergedResult.mergeWith(KeyListResult(startKeyListing(GpgME::OpenPGP)));
     m_mergedResult.mergeWith(KeyListResult(startKeyListing(GpgME::CMS)));
 
-    if (m_jobsPending.size() != 0) {
+    if (!m_jobsPending.empty()) {
         return;
     }
 
@@ -1671,41 +1747,13 @@ bool allKeysAllowUsage(const T &keys, KeyCache::KeyUsage usage)
     case KeyCache::KeyUsage::AnyUsage:
         return true;
     case KeyCache::KeyUsage::Sign:
-        return std::all_of(std::begin(keys),
-                           std::end(keys),
-#if GPGMEPP_KEY_HAS_HASCERTIFY_SIGN_ENCRYPT_AUTHENTICATE
-                           std::mem_fn(&Key::hasSign)
-#else
-                           Kleo::keyHasSign
-#endif
-        );
+        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::hasSign));
     case KeyCache::KeyUsage::Encrypt:
-        return std::all_of(std::begin(keys),
-                           std::end(keys),
-#if GPGMEPP_KEY_HAS_HASCERTIFY_SIGN_ENCRYPT_AUTHENTICATE
-                           std::mem_fn(&Key::hasEncrypt)
-#else
-                           Kleo::keyHasEncrypt
-#endif
-        );
+        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::hasEncrypt));
     case KeyCache::KeyUsage::Certify:
-        return std::all_of(std::begin(keys),
-                           std::end(keys),
-#if GPGMEPP_KEY_HAS_HASCERTIFY_SIGN_ENCRYPT_AUTHENTICATE
-                           std::mem_fn(&Key::hasCertify)
-#else
-                           Kleo::keyHasCertify
-#endif
-        );
+        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::hasCertify));
     case KeyCache::KeyUsage::Authenticate:
-        return std::all_of(std::begin(keys),
-                           std::end(keys),
-#if GPGMEPP_KEY_HAS_HASCERTIFY_SIGN_ENCRYPT_AUTHENTICATE
-                           std::mem_fn(&Key::hasAuthenticate)
-#else
-                           Kleo::keyHasAuthenticate
-#endif
-        );
+        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::hasAuthenticate));
     }
     qCDebug(LIBKLEO_LOG) << __func__ << "called with invalid usage" << int(usage);
     return false;
@@ -1761,5 +1809,10 @@ void KeyCache::setGroups(const std::vector<KeyGroup> &groups)
     Q_EMIT keysMayHaveChanged();
 }
 
+std::vector<CardKeyStorageInfo> KeyCache::cardsForSubkey(const GpgME::Subkey &subkey) const
+{
+    return d->m_cards[QByteArray(subkey.keyGrip())];
+}
+
 #include "moc_keycache.cpp"
 #include "moc_keycache_p.cpp"
