@@ -170,17 +170,12 @@ private:
     const unsigned int mKeyUsage;
 };
 
-static QString iconPath(const QString &name)
-{
-    return QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("libkleopatra/pics/") + name + QStringLiteral(".png"));
-}
-
 ColumnStrategy::ColumnStrategy(unsigned int keyUsage)
     : KeyListView::ColumnStrategy()
-    , mKeyGoodPix(iconPath(QStringLiteral("key_ok")))
-    , mKeyBadPix(iconPath(QStringLiteral("key_bad")))
-    , mKeyUnknownPix(iconPath(QStringLiteral("key_unknown")))
-    , mKeyValidPix(iconPath(QStringLiteral("key")))
+    , mKeyGoodPix(QStringLiteral(":/libkleopatra/key_ok"))
+    , mKeyBadPix(QStringLiteral(":/libkleopatra/key_bad"))
+    , mKeyUnknownPix(QStringLiteral(":/libkleopatra/key_unknown"))
+    , mKeyValidPix(QStringLiteral(":/libkleopatra/key"))
     , mKeyUsage(keyUsage)
 {
     if (keyUsage == 0) {
@@ -217,8 +212,8 @@ QString ColumnStrategy::text(const GpgME::Key &key, int col) const
 {
     switch (col) {
     case 0: {
-        if (key.shortKeyID()) {
-            return Formatting::prettyID(key.shortKeyID());
+        if (key.keyID()) {
+            return Formatting::prettyID(key.keyID());
         } else {
             return xi18n("<placeholder>unknown</placeholder>");
         }
@@ -240,8 +235,8 @@ QString ColumnStrategy::accessibleText(const GpgME::Key &key, int col) const
 {
     switch (col) {
     case 0: {
-        if (key.shortKeyID()) {
-            return Formatting::accessibleHexID(key.shortKeyID());
+        if (key.keyID()) {
+            return Formatting::accessibleHexID(key.keyID());
         }
         [[fallthrough]];
     }
@@ -261,7 +256,7 @@ QString ColumnStrategy::toolTip(const GpgME::Key &key, int) const
     QString keyStatusString;
     if (!checkKeyUsage(key, mKeyUsage, &keyStatusString)) {
         // Show the status in bold if there is a problem
-        keyStatusString = QLatin1StringView("<b>") % keyStatusString % QLatin1String("</b>");
+        keyStatusString = QLatin1StringView("<b>") % keyStatusString % QLatin1StringView("</b>");
     }
 
     QString html = QStringLiteral("<qt><p style=\"style='white-space:pre'\">");
@@ -462,7 +457,7 @@ void KeySelectionDialog::setUpUI(Options options, const QString &initialQuery)
     mTopLayout->addWidget(mKeyListView, 10);
 
     if (options & RememberChoice) {
-        mRememberCB = new QCheckBox(i18n("&Remember choice"), page);
+        mRememberCB = new QCheckBox(i18nc("@option:check", "&Remember choice"), page);
         mTopLayout->addWidget(mRememberCB);
         mRememberCB->setWhatsThis(
             i18n("<qt><p>If you check this box your choice will "
@@ -646,7 +641,7 @@ void KeySelectionDialog::slotRereadKeys()
         KMessageBox::information(this,
                                  i18n("No backends found for listing keys. "
                                       "Check your installation."),
-                                 i18n("Key Listing Failed"));
+                                 i18nc("@title:window", "Key Listing Failed"));
         connectSignals();
     }
 }
@@ -664,7 +659,7 @@ void KeySelectionDialog::slotStartCertificateManager(const QString &query)
         KMessageBox::error(this,
                            i18n("Could not start certificate manager; "
                                 "please check your installation."),
-                           i18n("Certificate Manager Error"));
+                           i18nc("@title:window", "Certificate Manager Error"));
     } else {
         QProcess::startDetached(QStringLiteral("kleopatra"), args);
         qCDebug(KLEO_UI_LOG) << "\nslotStartCertManager(): certificate manager started.";
@@ -682,7 +677,7 @@ static void showKeyListError(QWidget *parent, const GpgME::Error &err)
         "<p><b>%1</b></p></qt>",
         Formatting::errorAsString(err));
 
-    KMessageBox::error(parent, msg, i18n("Key Listing Failed"));
+    KMessageBox::error(parent, msg, i18nc("@title:window", "Key Listing Failed"));
 }
 #endif // __KLEO_UI_SHOW_KEY_LIST_ERROR_H__
 
@@ -943,15 +938,13 @@ void KeySelectionDialog::slotFilter()
     }
 
     // OK, so we need to filter:
-    QRegularExpression keyIdRegExp(QRegularExpression::anchoredPattern(QLatin1StringView("(?:0x)?[A-F0-9]{1,8}")), QRegularExpression::CaseInsensitiveOption);
+    QRegularExpression keyIdRegExp(QRegularExpression::anchoredPattern(QLatin1StringView("(?:0x)?[A-F0-9]{1,16}")), QRegularExpression::CaseInsensitiveOption);
     if (keyIdRegExp.match(mSearchText).hasMatch()) {
-        if (mSearchText.startsWith(QLatin1StringView("0X")))
-        // search for keyID only:
-        {
+        if (mSearchText.startsWith(QLatin1StringView("0X"))) {
+            // search for keyID only:
             filterByKeyID(mSearchText.mid(2));
-        } else
-        // search for UID and keyID:
-        {
+        } else {
+            // search for UID and keyID:
             filterByKeyIDOrUID(mSearchText);
         }
     } else {
@@ -962,13 +955,13 @@ void KeySelectionDialog::slotFilter()
 
 void KeySelectionDialog::filterByKeyID(const QString &keyID)
 {
-    Q_ASSERT(keyID.length() <= 8);
+    Q_ASSERT(keyID.length() <= 16);
     Q_ASSERT(!keyID.isEmpty()); // regexp in slotFilter should prevent these
     if (keyID.isEmpty()) {
         showAllItems();
     } else {
         for (KeyListViewItem *item = mKeyListView->firstChild(); item; item = item->nextSibling()) {
-            item->setHidden(!item->text(0).toUpper().startsWith(keyID));
+            item->setHidden(!item->text(0).remove(u' ').toUpper().startsWith(keyID));
         }
     }
 }
@@ -996,7 +989,7 @@ void KeySelectionDialog::filterByKeyIDOrUID(const QString &str)
     QRegularExpression rx(QLatin1StringView("\\b") + QRegularExpression::escape(str), QRegularExpression::CaseInsensitiveOption);
 
     for (KeyListViewItem *item = mKeyListView->firstChild(); item; item = item->nextSibling()) {
-        item->setHidden(!item->text(0).toUpper().startsWith(str) && !anyUIDMatches(item, rx));
+        item->setHidden(!item->text(0).remove(u' ').toUpper().startsWith(str) && !anyUIDMatches(item, rx));
     }
 }
 
