@@ -12,18 +12,18 @@
 
 #include "scdaemon.h"
 
-#include "utils/assuan.h"
-#include "utils/hex.h"
-#include "utils/stringutils.h"
+#include "assuan.h"
+#include "hex.h"
+#include "stringutils.h"
+
+#include <libkleo_debug.h>
 
 #if __has_include(<QGpgME/Debug>)
-# include <QGpgME/Debug>
+#include <QGpgME/Debug>
 #endif
 
 #include <gpgme++/context.h>
 
-#include "libkleo_debug.h"
-
 using namespace Kleo;
 using namespace GpgME;
 
@@ -46,9 +46,7 @@ std::vector<std::string> Kleo::SCDaemon::getReaders(Error &err)
 
     result = split(readers, '\n');
     // remove empty entries; in particular, the last entry
-    result.erase(std::remove_if(std::begin(result), std::end(result),
-                                std::mem_fn(&std::string::empty)),
-                 std::end(result));
+    result.erase(std::remove_if(std::begin(result), std::end(result), std::mem_fn(&std::string::empty)), std::end(result));
 
     return result;
 }
